:-include(appendixA).

/*C. Transformation rules from REA (G&Mc,2000) to REAP*/
/*C.0. Auxiliary Statements: Mapping*/
/*mapping(control,is,participation).*/
mapping(cash_receipt,is,cash-receipt).
mapping(customer,is,buyer).
mapping(vendor,is,seller).
mapping(duality,is,transfer-duality).

/*C.1. Auxiliary Statements: Data Access*/
access('customer0001',to,'me',data).
access('customer0002',to,'me',data).
access('0000',to,'me',data).
access('0000',to,'customer0001',data).
access('0000',to,'customer0002',data).

/*C.X. Auxiliary Statements: Complexity reduction*/
is_a(X,is_a,Y,to,P):-rea_role(X,is_a,Y,to,P).
is_a(X,is_a,Y,to,P):-rea_relator(X,is_a,Y,to,P).


/*C.2. Redefining the Primitives*/
rea_role(Y,is_a,Z,to,'me'):-
	value(Z,X,_,_),
	rea_lattice(Z,_,_),
	atom_concat(Z,X,Y).
rea_role(Y,is_a,Z,to,'me'):-
	value(Alpha,X,_,_),
	mapping(Alpha,is,Z),
	atom_concat(Alpha,X,Y).
/*C.2.1. Selfdeclaration*/
rea_role('me',is_a,trading-partner,to,_).
rea_role('me',is_a,seller,to,X):-
    value(customer,X,_,_).
/*C.3. Redefining the Associations*/
/*C.3.1 Mapping with REA2 concept*/
rea_relator([X,Y],is_a,Z,to,'me'):-
	relpart(Beta,Gamma,Alpha,Delta,Eta),
	atom_concat(Beta,Gamma,X),
	atom_concat(Delta,Eta,Y),
	rea(Alpha,Phi),
	mapping(Phi,is,Z).
/*C.3.2. Is a REA2 concept*/
rea_relator([X,Y],is_a,Z,to,'me'):-
	relpart(Beta,Gamma,Alpha,Delta,Eta),
	atom_concat(Beta,Gamma,X),
	atom_concat(Delta,Eta,Y),
	rea(Alpha,Z),
	blockchain_interface(Z).
/*C.3.3. Needs more detail*/
rea_relator([X,Y],is_a,Omega,to,'me'):-
	relpart(Beta,Gamma,Alpha,Delta,Eta),
        event_pattern(Beta,is_a,_,Z),
        rea_relation(Z,mirrors,Omega),
	atom_concat(Beta,Gamma,X),
	atom_concat(Delta,Eta,Y),
	rea(Alpha,control).
rea_relator([X,Y],is_a,Omega,to,'me'):-
	relpart(Beta,Gamma,Alpha,Delta,Eta),
        mapping(Beta,is,Theta),
        event_pattern(Theta,is_a,_,Z),
        rea_relation(Z,mirrors,Omega),
	atom_concat(Beta,Gamma,X),
	atom_concat(Delta,Eta,Y),
	rea(Alpha,control).

/*C.I. Intentional Reasoning Paper*/
/*C.I.1. Conceptual Schema Definition*/
/*C.I.1.1. Entity Definitions*/
/*OMITTED*/

/*C.I.1.2. Relationship Definitions*/
/*OMITTED*/

/*C.I.1.3. Attribute Definitions*/
/*OMITTED*/

/*C.I.1.4. Constraint Definitions*/
/*OMITTED*/

/*C.I.1.5. Role Definitions*/
description(customer,customer_nr,id).
description(product,product_nr,id).
description(cash,cash_type,id).
description(sale,sale_nr,id).
description(cash_receipt,cash_receipt_nr,id).
/*C.I.1.5.X. Addition to the data according to fig. 7 in G&M2000*/
description(vendor,vendor_nr,id).
description(purchase,purchase_nr,id).
description(cash_disbursment,cash_disbursment_nr,id).

/*C.I.2. Accounting-Specific Classifications*/
/*C.I.2.1. REA Classifications*/
rea(sale,event).
rea(cash_receipt,event).
rea(customer,agent).
rea(product,resource).
rea(cash,resource).
rea(r1,control).
rea(r2,control).
rea(r3,duality).
rea(r4,stock-flow).
rea(r5,stock-flow).
/*Potentially incoherent*/
rea(sale,outflow).
rea(cash_receipt,inflow).

/*C.I.3. REA-based Definitions of Accounting Phenomena*/
/*OMITTED*/


/*C.I.4. Database*/
value(customer,'0001',customer_nr,'0001').
value(customer,'0001',customer_name,'Mead').
value(customer,'0002',customer_nr,'0002').
value(customer,'0002',customer_name,'Grabski').
value(sale,'0001',sale_nr,'0001').
value(sale,'0001',sale_amount,1000).
value(sale,'0001',sale_date,'07/10/95').
value(sale,'0002',sale_nr,'0002').
value(sale,'0002',sale_amount,1000).
value(sale,'0002',sale_date,'07/11/95').
value(sale,'0003',sale_nr,'0003').
value(sale,'0003',sale_amount,1250).
value(sale,'0003',sale_date,'07/13/95').
value(cash_receipt,'0001',cash_receipt_nr,'0001').
value(cash_receipt,'0001',cash_receipt_amount,1000).
value(cash_receipt,'0001',cash_receipt_date,'07/11/95').
value(cash_receipt,'0002',cash_receipt_nr,'0002').
value(cash_receipt,'0002',cash_receipt_amount,1250).
value(cash_receipt,'0002',cash_receipt_date,'07/12/95').
value(r1,r11,sale,'0001').
value(r1,r11,customer,'0001').
value(r1,r12,sale,'0002').
value(r1,r12,customer,'0001').
value(r1,r13,sale,'0003').
value(r1,r13,customer,'0002').
value(r2,r21,customer,'0001').
value(r2,r21,cash_receipt,'0001').
value(r2,r22,customer,'0001').
value(r2,r22,cash_receipt,'0002').
value(r3,r31,sale,'0001').
value(r3,r31,cash_receipt,'0001').
value(r3,r32,sale,'0002').
value(r3,r33,cash_receipt,'0002').
value(r3,r34,sale,'0003').
/*C.I.4.X. Addition to the data according to fig. 7 in G&M2000*/


/*C.I.5. Supportive Definitions*/
id(Object,IdAttribute) :-
  description(Object,IdAttribute,id).
occurrence(Object,Value) :-
  id(Object,IdAttribute),
  value(Object,_,IdAttribute,Value).
part(Object,Relationship,RelationshipValue,Value) :-
  occurrence(Object,Value),
  value(Relationship,RelationshipValue,Object,Value).
relpart(Object1,Object1Value,Relationship,Object2,Object2Value) :-
  part(Object1,Relationship,RelationshipValue,Object1Value),
  part(Object2,Relationship,RelationshipValue,Object2Value),
  not(Object1 == Object2).
/*Other definitions omitted*/
















